﻿/// @file config.cpp
///
/// @brief 設定ファイル入出力
///
#include "config.h"
#include "wx/filename.h"
#include "wx/fileconf.h"

Config::Config()
{
	ini_file = _T("");

	wav_file_path = _T("");
	sample_rate = 3;
	sample_bits = 0;
	baud = 0;
	auto_baud = true;
}

Config::~Config()
{
}

void Config::SetFileName(const wxString &file)
{
	ini_file = file;
}

void Config::Load()
{
	if (ini_file.IsEmpty()) return;

	// load ini file
	wxFileConfig *ini = new wxFileConfig(wxEmptyString,wxEmptyString,ini_file,wxEmptyString
		,wxCONFIG_USE_LOCAL_FILE | wxCONFIG_USE_RELATIVE_PATH | wxCONFIG_USE_NO_ESCAPE_CHARACTERS);

	int lv;
	ini->Read(_T("Path"), &wav_file_path);
	if (!ini->Read(_T("SampleRate"), &lv)) lv = sample_rate;
	if (lv < 0 || 3 < lv) lv = sample_rate;
	sample_rate = lv;
	if (!ini->Read(_T("SampleBits"), &lv)) lv = sample_bits;
	if (lv < 0 || 1 < lv) lv = sample_bits;
	sample_bits = lv;
	if (!ini->Read(_T("Baud"), &lv)) lv = baud;
	if (lv < 0 || 3 < lv) lv = baud;
	baud = lv;
	ini->Read(_T("AutoBaud"), &auto_baud);

	delete ini;
}

void Config::Load(const wxString &file)
{
	SetFileName(file);
	Load();
}

void Config::Save()
{
	if (ini_file.IsEmpty()) return;

	// save ini file
	wxFileConfig *ini = new wxFileConfig(wxEmptyString,wxEmptyString,ini_file,wxEmptyString
		,wxCONFIG_USE_LOCAL_FILE | wxCONFIG_USE_RELATIVE_PATH | wxCONFIG_USE_NO_ESCAPE_CHARACTERS);
	ini->Write(_T("Path"), wav_file_path);
	ini->Write(_T("SampleRate"), sample_rate);
	ini->Write(_T("SampleBits"), sample_bits);
	ini->Write(_T("Baud"), baud);
	ini->Write(_T("AutoBaud"), auto_baud);
	// write
	delete ini;
}

