﻿/// @file maddressbox.cpp
///
/// @brief 開始アドレス設定ダイアログ
///
#include "maddressbox.h"

BEGIN_EVENT_TABLE(MAddressBox, wxDialog)
	EVT_BUTTON(wxID_OK, MAddressBox::OnClickOk)
END_EVENT_TABLE()

MAddressBox::MAddressBox()
{
	MAddressBox(NULL, wxID_ANY);
}

MAddressBox::MAddressBox(wxWindow* parent, wxWindowID id)
	: wxDialog(parent, id, _("Machine code information"), wxDefaultPosition, wxDefaultSize, wxDEFAULT_DIALOG_STYLE)
{
	wxSizerFlags flags = wxSizerFlags().Expand().Border(wxALL, 4);

	wxBoxSizer  *szrAll = new wxBoxSizer(wxVERTICAL);
	wxGridSizer *gszr;

	lblTitle = new wxStaticText(this, IDC_STA_TITLE, _T(""));
	szrAll->Add(lblTitle, flags);

	gszr = new wxFlexGridSizer(3, 3, 0, 0);
	gszr->Add(new wxStaticText(this, wxID_ANY, _("Start address")), flags);
	gszr->Add(new wxStaticText(this, wxID_ANY, _T("&&H")), flags);
	txtStartAddr = new wxTextCtrl(this, IDC_START_ADDR);
	gszr->Add(txtStartAddr, flags);
	gszr->Add(new wxStaticText(this, wxID_ANY, _("Execute address")), flags);
	gszr->Add(new wxStaticText(this, wxID_ANY, _T("&&H")), flags);
	txtExecAddr = new wxTextCtrl(this, IDC_EXEC_ADDR);
	gszr->Add(txtExecAddr, flags);
	gszr->Add(new wxStaticText(this, wxID_ANY, _("Data size")), flags);
	gszr->Add(new wxStaticText(this, wxID_ANY, _T("&&H")), flags);
	txtDataSize = new wxTextCtrl(this, IDC_DATA_SIZE);
	gszr->Add(txtDataSize, flags);
	szrAll->Add(gszr, flags);

	chkIncHeader = new wxCheckBox(this, IDC_INC_HEADER, _("Include header and footer"));
	szrAll->Add(chkIncHeader, flags);

	lblNoHeader = new wxStaticText(this, IDC_NO_HEADER, _("If cancel, file will output without header."));
	szrAll->Add(lblNoHeader, flags);

	wxSizer *szrButtons = CreateButtonSizer(wxOK|wxCANCEL);
	szrAll->Add(szrButtons, flags);

	SetSizerAndFit(szrAll);

	// initialize
	start_addr = 0;
	exec_addr  = 0;
	data_size  = 0;
	valid      = false;
}

// ダイアログ表示
int MAddressBox::showMAddressBox(bool hide_no_header_info)
{
	enable_cancel = true;
	
	if (hide_no_header_info) {
		lblNoHeader->Show(false);
	} else {
		lblNoHeader->Show(true);
	}
	init_dialog();
	int rc = wxDialog::ShowModal();
	if (rc == wxID_OK) {
		term_dialog();
		return 1;
	}
	return 0;
}

void MAddressBox::init_dialog()
{
	wxString buf = _T("");

	if (valid) buf.Printf(_T("%04lx"), (long)start_addr);
	txtStartAddr->SetValue(buf);
	txtStartAddr->SetMaxLength(4);

	if (valid) buf.Printf(_T("%04lx"), (long)exec_addr);
	txtExecAddr->SetValue(buf);
	txtExecAddr->SetMaxLength(4);

	buf.Printf(_T("%04lx"), (long)data_size);
	txtDataSize->SetValue(buf);
	txtDataSize->SetMaxLength(4);
	txtDataSize->Enable(false);


	wxWindow *cancelBtn = FindWindowById(wxID_CANCEL, this);
	if (cancelBtn != NULL) {
		if (enable_cancel) {
			cancelBtn->Enable(true);
		} else {
			cancelBtn->Enable(false);
		}
	}

	chkIncHeader->SetValue(include_header);
	if (valid) {
		lblTitle->SetLabel(_("Please confirm this information."));	// 情報を確認して下さい。
		chkIncHeader->Enable(true);
	} else {
		lblTitle->SetLabel(_("Please input some information."));	// 情報を入力してください。
		chkIncHeader->Enable(false);
	}
}


void MAddressBox::term_dialog()
{
}

void MAddressBox::OnClickOk(wxCommandEvent& event)
{
	bool rc = true;
	wxString buf;

	// テキスト
	buf = txtStartAddr->GetValue();
	rc = chk_addr(buf);
	if (rc != true) return;
	buf.ToLong(&start_addr, 16);

	buf = txtExecAddr->GetValue();
	rc = chk_addr(buf);
	if (rc != true) return;
	buf.ToLong(&exec_addr, 16);

	valid = true;
	EndModal(wxID_OK);
}

void MAddressBox::setValid(bool val)
{
	valid = val;
	if (!valid) {
		start_addr = 0;
		exec_addr  = 0;
		data_size  = 0;
		include_header = false;
	}
}

bool MAddressBox::chk_addr(wxString &str)
{
	if (str.Length() <= 0 || str.Length() > 4 || !chk_str(str)) {
		wxMessageBox(_("Address accepts only 4 digits hexadecimal.") // アドレスは16進数4桁で入力してください。
			, _("error"), wxOK | wxICON_ERROR, this);
		return false;
	}
	return true;
}

bool MAddressBox::chk_str(wxString &str)
{
	bool rc = true;
	UINT len;

	len = str.Length();

	if (len <= 0) {
			rc = false;
	}
	for(UINT i=0; i < len; i++) {
		if (!_istxdigit(str.GetChar(i))) {	// hex str
			rc = false;
			break;
		}
	}
	return rc;
}
