﻿/// @file wavtool.cpp
///
/// wavtool.cpp
///
#include "parsewav.h"
#include "wavtool.h"
#include "configbox.h"
#include "wx/filename.h"
#include "res/wavtool.xpm"

IMPLEMENT_APP(WavtoolApp)

bool WavtoolApp::OnInit()
{
	SetAppPath();
	SetAppName(_T("wavtool"));

	// load ini file
	mConfig.Load(ini_path + GetAppName() + _T(".ini"));

	// set locale search path and catalog name
	mLocale.AddCatalogLookupPathPrefix(res_path + _T("lang"));
	mLocale.AddCatalogLookupPathPrefix(_T("lang"));
	mLocale.AddCatalog(_T("wavtool"));

	if (!wxApp::OnInit()) {
		return false;
	}

	WavtoolFrame *frame = new WavtoolFrame(GetAppName(), wxSize(480, 400) );
	frame->Show(true);
	SetTopWindow(frame);
	return true;
}

int WavtoolApp::OnExit()
{
	// save ini file
	mConfig.Save();

	return 0;
}

void WavtoolApp::SetAppPath()
{
	app_path = wxFileName::FileName(argv[0]).GetPath(wxPATH_GET_SEPARATOR);
#ifdef __WXOSX__
	if (app_path.Find(_T("MacOS")) >= 0) {
		wxFileName file = wxFileName::FileName(app_path+"../../");
		file.Normalize();
		ini_path = file.GetPath(wxPATH_GET_SEPARATOR);
		res_path = ini_path+_T("Contents/Resources/");
	} else
#endif
	{
		ini_path = app_path;
		res_path = app_path;
	}
}

const wxString &WavtoolApp::GetAppPath()
{
	return app_path;
}

const wxString &WavtoolApp::GetIniPath()
{
	return ini_path;
}

const wxString &WavtoolApp::GetResPath()
{
	return res_path;
}

Config *WavtoolApp::GetConfig()
{
	return &mConfig;
}

//
// Frame
//
// Attach Event
BEGIN_EVENT_TABLE(WavtoolFrame, wxFrame)
	// menu event
	EVT_MENU(wxID_EXIT,  WavtoolFrame::OnQuit)
	EVT_MENU(wxID_ABOUT, WavtoolFrame::OnAbout)

	EVT_MENU(IDM_OPEN_FILE, WavtoolFrame::OnOpenFile)
	EVT_MENU(IDM_CLOSE_FILE, WavtoolFrame::OnCloseFile)

	EVT_MENU(IDM_EXPORT_REAL, WavtoolFrame::OnExportFile)
	EVT_MENU(IDM_EXPORT_L3,   WavtoolFrame::OnExportFile)
	EVT_MENU(IDM_EXPORT_T9X,  WavtoolFrame::OnExportFile)
	EVT_MENU(IDM_EXPORT_L3B,  WavtoolFrame::OnExportFile)
	EVT_MENU(IDM_EXPORT_L3C,  WavtoolFrame::OnExportFile)
	EVT_MENU(IDM_EXPORT_WAV,  WavtoolFrame::OnExportFile)

	EVT_MENU(IDM_ANALYZE_WAV, WavtoolFrame::OnAnalyzeWave)
	EVT_MENU(IDM_ANALYZE_FILES, WavtoolFrame::OnAnalyzeFiles)

	EVT_MENU(IDM_SETS_RFTYPE,  WavtoolFrame::OnSetsRftype)
	EVT_MENU(IDM_SETS_MACHINE, WavtoolFrame::OnSetsMachine)

	EVT_MENU(IDM_SETS_BAUD_AUTO, WavtoolFrame::OnSetsBaudRate)
	EVT_MENU(IDM_SETS_BAUD_600, WavtoolFrame::OnSetsBaudRate)
	EVT_MENU(IDM_SETS_BAUD_1200, WavtoolFrame::OnSetsBaudRate)
	EVT_MENU(IDM_SETS_BAUD_2400, WavtoolFrame::OnSetsBaudRate)
	EVT_MENU(IDM_SETS_BAUD_300, WavtoolFrame::OnSetsBaudRate)

	EVT_MENU(IDM_SETS_CORRECT_NONE, WavtoolFrame::OnSetsCorrectType)
	EVT_MENU(IDM_SETS_CORRECT_COS, WavtoolFrame::OnSetsCorrectType)
	EVT_MENU(IDM_SETS_CORRECT_SIN, WavtoolFrame::OnSetsCorrectType)

	EVT_MENU(IDM_SETS_CONFIGURE,  WavtoolFrame::OnConfigure)

//	EVT_MENU_OPEN(WavtoolFrame::OnMenuOpen)

//	EVT_DROP_FILES(WavtoolFrame::OnDropFiles)
END_EVENT_TABLE()

WavtoolFrame::WavtoolFrame(const wxString& title, const wxSize& size)
       : wxFrame(NULL, -1, title, wxDefaultPosition, size)
{
	// icon
#ifdef __WXMSW__
	SetIcon(wxIcon(_T("wavtool")));
#elif defined(__WXGTK__) || defined(__WXMOTIF__)
	SetIcon(wxIcon(wavtool_xpm));
#endif

	// menu
	menuFile = new wxMenu;
	menuSets = new wxMenu;
	menuHelp = new wxMenu;
	wxMenu *smenu;

	menuFile->Append( IDM_OPEN_FILE, _("&Open...") );
	menuFile->Append( IDM_CLOSE_FILE, _("&Close") );
	menuFile->AppendSeparator();
	smenu = new wxMenu;
	smenu->Append( IDM_EXPORT_REAL, _("&Real File...") );
	smenu->AppendSeparator();
	smenu->Append( IDM_EXPORT_L3, _("&L3 File...") );
	smenu->Append( IDM_EXPORT_T9X, _("&T9X File...") );
	smenu->Append( IDM_EXPORT_L3B, _("L3&B File...") );
	smenu->Append( IDM_EXPORT_L3C, _("L3&C File...") );
	smenu->Append( IDM_EXPORT_WAV, _("&WAV File...") );
	menuFile->Append( IDM_EXPORT, _("&Export To"), smenu );
	menuFile->AppendSeparator();
	menuFile->Append( IDM_ANALYZE_WAV, _("&Analyze Wave") );
	menuFile->Append( IDM_ANALYZE_FILES, _("Analyze &Files") );
	menuFile->AppendSeparator();
	menuFile->Append( wxID_EXIT, _("E&xit") );
	// settings menu
	menuSets->Append( IDM_SETS_RFTYPE, _("Data &Type...") );
	menuSets->Append( IDM_SETS_MACHINE, _("&Machine Code...") );
	menuSets->AppendSeparator();
	smenu = new wxMenu;
	smenu->AppendCheckItem( IDM_SETS_BAUD_AUTO, _("&Auto Detect") );
	smenu->AppendSeparator();
	smenu->AppendRadioItem( IDM_SETS_BAUD_600, _("600") );
	smenu->AppendRadioItem( IDM_SETS_BAUD_1200, _("1200") );
	smenu->AppendRadioItem( IDM_SETS_BAUD_2400, _("2400") );
	smenu->AppendRadioItem( IDM_SETS_BAUD_300, _("300") );
	menuSets->Append( IDM_SETS_BAUD, _("&Baud Rate"), smenu );
	menuSets->AppendSeparator();
	smenu = new wxMenu;
	smenu->AppendRadioItem( IDM_SETS_CORRECT_NONE, _("None") );
	smenu->AppendRadioItem( IDM_SETS_CORRECT_COS, _("COS Wave") );
	smenu->AppendRadioItem( IDM_SETS_CORRECT_SIN, _("SIN Wave") );
	menuSets->Append( IDM_SETS_CORRECT, _("Wave &Correct"), smenu );
	menuSets->AppendSeparator();
	menuSets->Append( IDM_SETS_CONFIGURE, _("&Details...") );
	// help menu
	menuHelp->Append( wxID_ABOUT, _("&About...") );

	// menu bar
	wxMenuBar *menuBar = new wxMenuBar;
	menuBar->Append( menuFile, _("&File") );
	menuBar->Append( menuSets, _("&Settings") );
	menuBar->Append( menuHelp, _("&Help") );

	SetMenuBar( menuBar );

	// control panel
	panel = new WavtoolPanel(this);

	//
//	DragAcceptFiles(true);

	// initialize
	wav = new PARSEWAV(this);
	wav->SetLogBufferPtr(&text_buffer);

	cfgbox = new ConfigBox(this, IDD_CONFIGBOX);

	// load ini file
	Config *ini = wxGetApp().GetConfig();
	wav_file_path = ini->wav_file_path;
	wav->SetSampleRate(ini->sample_rate);
	wav->SetSampleBits(ini->sample_bits);
	wav->SetBaud(ini->baud);
	wav->SetAutoBaud(ini->auto_baud);

	// update menu
	UpdateMenu(menuFile);
	UpdateMenu(menuSets);
}

WavtoolFrame::~WavtoolFrame()
{
	// save ini file
	Config *ini = wxGetApp().GetConfig();
	ini->wav_file_path = wav_file_path;
	ini->sample_rate = wav->GetSampleRate();
	ini->sample_bits = wav->GetSampleBits();
	ini->baud = wav->GetBaud();
	ini->auto_baud = (wav->GetAutoBaud() != 0);

	delete cfgbox;
	delete wav;
}

/// メニュー更新
void WavtoolFrame::OnMenuOpen(wxMenuEvent& event)
{
	wxMenu *menu = event.GetMenu();

	if (menu == NULL) return;

	UpdateMenu(menu);
}

/// メニュー更新
void WavtoolFrame::UpdateMenu(wxMenu* menu)
{
	int i;
	if (menu == menuFile) {	// File...
		// Export to ...
		menu->Enable(IDM_CLOSE_FILE,  false);
		menu->Enable(IDM_EXPORT_L3,   false);
		menu->Enable(IDM_EXPORT_T9X,  false);
		menu->Enable(IDM_EXPORT_L3B,  false);
		menu->Enable(IDM_EXPORT_L3C,  false);
		menu->Enable(IDM_EXPORT_WAV,  false);
		menu->Enable(IDM_EXPORT_REAL, false);
		menu->Enable(IDM_ANALYZE_WAV, false);
		menu->Enable(IDM_ANALYZE_FILES, false);

		if (wav->IsOpenedDataFile()) {
			menu->Enable(IDM_CLOSE_FILE, true);

			switch(wav->GetDataFileType()) {
			case PARSEWAV::WAV:	//wav
					menu->Enable(IDM_EXPORT_L3,   true);
					menu->Enable(IDM_EXPORT_T9X,  true);
					menu->Enable(IDM_EXPORT_L3B,  true);
					menu->Enable(IDM_EXPORT_L3C,  true);
					menu->Enable(IDM_EXPORT_WAV,  true);
					menu->Enable(IDM_EXPORT_REAL, true);
					menu->Enable(IDM_ANALYZE_WAV, true);
					menu->Enable(IDM_ANALYZE_FILES, true);
					break;
				case PARSEWAV::L3C:	//l3c
					menu->Enable(IDM_EXPORT_L3,   true);
					menu->Enable(IDM_EXPORT_T9X,  true);
					menu->Enable(IDM_EXPORT_L3B,  true);
					menu->Enable(IDM_EXPORT_WAV,  true);
					menu->Enable(IDM_EXPORT_REAL, true);
					menu->Enable(IDM_ANALYZE_FILES, true);
					break;
				case PARSEWAV::L3B:	//l3b
					menu->Enable(IDM_EXPORT_L3,   true);
					menu->Enable(IDM_EXPORT_T9X,  true);
					menu->Enable(IDM_EXPORT_L3C,  true);
					menu->Enable(IDM_EXPORT_WAV,  true);
					menu->Enable(IDM_EXPORT_REAL, true);
					menu->Enable(IDM_ANALYZE_FILES, true);
					break;
				case PARSEWAV::T9X:	//t9x
					menu->Enable(IDM_EXPORT_L3,   true);
					menu->Enable(IDM_EXPORT_L3B,  true);
					menu->Enable(IDM_EXPORT_L3C,  true);
					menu->Enable(IDM_EXPORT_WAV,  true);
					menu->Enable(IDM_EXPORT_REAL, true);
					menu->Enable(IDM_ANALYZE_FILES, true);
					break;
				case PARSEWAV::L3:	//l3
					menu->Enable(IDM_EXPORT_T9X,  true);
					menu->Enable(IDM_EXPORT_L3B,  true);
					menu->Enable(IDM_EXPORT_L3C,  true);
					menu->Enable(IDM_EXPORT_WAV,  true);
					menu->Enable(IDM_EXPORT_REAL, true);
					menu->Enable(IDM_ANALYZE_FILES, true);
					break;
				case PARSEWAV::REAL:	//real
					menu->Enable(IDM_EXPORT_L3,   true);
					menu->Enable(IDM_EXPORT_T9X,  true);
					menu->Enable(IDM_EXPORT_L3B,  true);
					menu->Enable(IDM_EXPORT_L3C,  true);
					menu->Enable(IDM_EXPORT_WAV,  true);
					break;
				default:
					break;
			}
		}
	} else if (menu == menuSets) {
		menu->Enable(IDM_SETS_RFTYPE,  false);
		menu->Enable(IDM_SETS_MACHINE, false);

		if (wav->GetDataFileType() == PARSEWAV::REAL) {
			menu->Enable(IDM_SETS_RFTYPE, true);
			if (wav->GetRfDataFormat() == 2) {
				menu->Enable(IDM_SETS_MACHINE, true);
			}
		}

		menu->Check(IDM_SETS_BAUD_AUTO, wav->GetAutoBaud() != 0);
		i = wav->GetBaud();
		menu->Check(IDM_SETS_BAUD_600 + i, true);

		if (wav->GetCorrect() != 0) {
			i = wav->GetCorrectType();
			menu->Check(IDM_SETS_CORRECT_COS + i, true);
		} else {
			menu->Check(IDM_SETS_CORRECT_NONE, true);
		}
	}
}

/// ドロップされたファイルを開く
#if 0
void WavtoolFrame::OnDropFiles(wxDropFilesEvent& event)
{
	wxString *names = event.GetFiles();
	int num = event.GetNumberOfFiles();

	// 最初のファイル名を取得
	if (num > 0) {
		CloseDataFile();
		OpenDataFile(names[0]);
	}
}
#endif
void WavtoolFrame::OpenDroppedFile(wxString &path)
{
	CloseDataFile();
	OpenDataFile(path);
}

void WavtoolFrame::OnQuit(wxCommandEvent& WXUNUSED(event))
{
 	CloseDataFile();
	Close(true);
}

void WavtoolFrame::OnAbout(wxCommandEvent& WXUNUSED(event))
{
	WavtoolAbout(this, wxID_ANY).ShowModal();
}

void WavtoolFrame::OnOpenFile(wxCommandEvent& WXUNUSED(event))
{
	WavtoolFileDialog *dlg = new WavtoolFileDialog(
		_("Open file"),
		wav_file_path,
		wxEmptyString,
		_("Supported files (*.wav;*.l3c;*.l3b;*.l3;*.t9x;*.bin;*.bas;*.dat)|*.wav;*.l3c;*.l3b;*.l3;*.t9x;*.bin;*.bas;*.dat|All files (*.*)|*.*"),
		wxFD_OPEN);

	int rc = dlg->ShowModal();
	wxString path = dlg->GetPath();

	delete dlg;

	if (rc == wxID_OK) {
		OpenDataFile(path);
	}
}

void WavtoolFrame::OnCloseFile(wxCommandEvent& WXUNUSED(event))
{
	CloseDataFile();
}

/// エクスポート
void WavtoolFrame::OnExportFile(wxCommandEvent& event)
{
	int id = event.GetId();

	ExportFile(id);
}

/// 波形解析
void WavtoolFrame::OnAnalyzeWave(wxCommandEvent& event)
{
	if (!wav->IsOpenedDataFile()) return;
	if (wav->GetDataFileType() != PARSEWAV::WAV) return;

	wav->AnalyzeWave();

	// disp infomation on the window
	panel->GetTextInfo()->SetValue(text_buffer);

	UpdateMenu(menuSets);
}

/// ファイル内容解析
void WavtoolFrame::OnAnalyzeFiles(wxCommandEvent& event)
{
	if (!wav->IsOpenedDataFile()) return;

	wav->ExportData(PARSEWAV::NO_FILE);

	// disp infomation on the window
	panel->GetTextInfo()->SetValue(text_buffer);
}

/// データ種類設定ダイアログ
void WavtoolFrame::OnSetsRftype(wxCommandEvent& WXUNUSED(event))
{
	if (!wav->IsOpenedDataFile()) return;
	if (wav->GetDataFileType() != PARSEWAV::REAL) return;

	bool rc;

	rc = wav->ShowRfTypeBox();
	if (!rc) {
		// cancel button or error
		return;
	}
}

/// マシン語設定ダイアログ
void WavtoolFrame::OnSetsMachine(wxCommandEvent& WXUNUSED(event))
{
	if (!wav->IsOpenedDataFile()) return;
	if (wav->GetDataFileType() != PARSEWAV::REAL) return;
	if (wav->GetRfDataFormat() != 2) return;

	wav->ShowMAddressBox(true);
}

/// ボーレート設定
void WavtoolFrame::OnSetsBaudRate(wxCommandEvent& event)
{
	int id = event.GetId();
	switch(id) {
	case IDM_SETS_BAUD_AUTO:
		wav->SetAutoBaud(event.IsChecked() ? 1 : 0);
		break;
	default:
		wav->SetBaud(id-IDM_SETS_BAUD_600);
		break;
	}
}

/// 波形補正設定
void WavtoolFrame::OnSetsCorrectType(wxCommandEvent& event)
{
	int id = event.GetId();
	switch(id) {
	case IDM_SETS_CORRECT_NONE:
		wav->SetCorrect(0);
		break;
	default:
		wav->SetCorrect(1);
		wav->SetCorrectType(id-IDM_SETS_CORRECT_COS);
		break;
	}
}

/// 設定ダイアログ
void WavtoolFrame::OnConfigure(wxCommandEvent& WXUNUSED(event))
{
	cfgbox->SetParam(wav->GetParam());

	cfgbox->ShowModal();

	wav->SetParam(cfgbox->GetParam());

	UpdateMenu(menuSets);
}

/// 指定したファイルを開く
void WavtoolFrame::OpenDataFile(wxString &path)
{
	bool rc;
	wxString title;

	wav_file_path = wxFileName::FileName(path).GetPath(wxPATH_GET_SEPARATOR);

	rc = wav->OpenDataFile(path);
	if (!rc) {
		// cancel button or error
		return;
	}

	// update window
	title = wxGetApp().GetAppName() + _T(" - ") + path;
	SetTitle(title);
	panel->GetTextName()->SetValue(path);
	panel->GetTextInfo()->SetValue(_T(""));

	UpdateMenu(menuFile);
	UpdateMenu(menuSets);
}

/// ファイルを閉じる
void WavtoolFrame::CloseDataFile()
{
	wav->CloseDataFile();

	// update window
	wxString title = wxGetApp().GetAppName();
	SetTitle(title);
	panel->GetTextName()->SetValue(_T(""));

	UpdateMenu(menuFile);
	UpdateMenu(menuSets);
}

/// エクスポート
void WavtoolFrame::ExportFile(int id)
{
	int rc;
	wxString file_base;
	wxString wild_card;
	bool enable = true;

	if (!wav->IsOpenedDataFile()) return;

	wav->GetFileNameBase(file_base);

	switch(id) {
		case IDM_EXPORT_L3:
			file_base += _T(".l3");
			wild_card = _("L3 file (*.l3)|*.l3");
			if (wav->GetDataFileType() == PARSEWAV::L3) enable = false;
			break;
		case IDM_EXPORT_T9X:
			file_base += _T(".t9x");
			wild_card = _("T9X file (*.t9x)|*.t9x");
			if (wav->GetDataFileType() == PARSEWAV::T9X) enable = false;
			break;
		case IDM_EXPORT_L3B:
			file_base += _T(".l3b");
			wild_card = _("L3B file (*.l3b)|*.l3b");
			if (wav->GetDataFileType() == PARSEWAV::L3B) enable = false;
			break;
		case IDM_EXPORT_L3C:
			file_base += _T(".l3c");
			wild_card = _("L3C file (*.l3c)|*.l3c");
			if (wav->GetDataFileType() == PARSEWAV::L3C) enable = false;
			break;
		case IDM_EXPORT_WAV:
			file_base += _T(".wav");
			wild_card = _("Wave file (*.wav)|*.wav");
			break;
		case IDM_EXPORT_REAL:
#if defined(__WXGTK__) || defined(__WXMOTIF__)
			file_base += _T(".bin");
			wild_card = _("Real file (*.bin)|*.bin|All Files (*.*)|*.*");
#else
			file_base += _T(".bin");
			wild_card = _("Real file (*.bin)|*.bin|BASIC file (*.bas)|*.bas|Data file (*.dat)|*.dat|All files (*.*)|*.*");
#endif
			if (wav->GetDataFileType() == PARSEWAV::REAL) enable = false;
			break;
		default:
			file_base += _T("");
			wild_card = _("All files (*.*)|*.*");
			break;
	}
	if (!enable) return;

	WavtoolFileDialog *dlg = new WavtoolFileDialog(
		_("Export file"),
		wav_file_path,
		file_base,
		wild_card,
		wxFD_SAVE | wxFD_OVERWRITE_PROMPT);

	rc = dlg->ShowModal();
	wxString path = dlg->GetPath();

	delete dlg;

	if (rc == wxID_OK) {
		bool rt = wav->OpenOutFile(path);
		if (!rt) {
			// cancel button or error
			return;
		}
		//
		rt = wav->ExportData();
		wav->CloseOutFile();
		if (!rt) {
			// cancel button or error
			return;
		}

		// disp infomation on the window
		panel->GetTextInfo()->SetValue(text_buffer);
	}
}

//
// Control Panel
//
// Attach Event
BEGIN_EVENT_TABLE(WavtoolPanel, wxPanel)
	// event
	EVT_SIZE(WavtoolPanel::OnSize)
END_EVENT_TABLE()

WavtoolPanel::WavtoolPanel(WavtoolFrame *parent)
       : wxPanel(parent)
{
	wxPoint pos;
	wxSize  frame_size = parent->GetSize();
	wxSize  size;
	long style;

	pos.x = 0; pos.y = 0;
	size.x = frame_size.x; size.y = -1;
	style = wxTE_READONLY;
	textName = new wxTextCtrl(this, IDC_TEXT_NAME, wxEmptyString, pos, size, style);
	wxSize      sizeName = textName->GetSize();

	pos.x = 0; pos.y = sizeName.y;
	size.x = frame_size.x; size.y = frame_size.y - sizeName.y;
	style = wxTE_READONLY | wxTE_MULTILINE;
	textInfo = new wxTextCtrl(this, IDC_TEXT_INFO, wxEmptyString, pos, size, style);

	// drag and drop
	textName->SetDropTarget(new WavtoolFileDropTarget(parent));
	textInfo->SetDropTarget(new WavtoolFileDropTarget(parent));
}

void WavtoolPanel::OnSize(wxSizeEvent& event)
{
	wxSize size = event.GetSize();

	textName->SetSize(size.x, textName->GetSize().y);
	textInfo->SetSize(size.x, size.y - textName->GetSize().y);
}

//
// File Dialog
//
WavtoolFileDialog::WavtoolFileDialog(const wxString& message, const wxString& defaultDir, const wxString& defaultFile, const wxString& wildcard, long style)
            : wxFileDialog(NULL, message, defaultDir, defaultFile, wildcard, style)
{
}

//
// File Drag and Drop
//
WavtoolFileDropTarget::WavtoolFileDropTarget(WavtoolFrame *parent)
			: frame(parent)
{
}

bool WavtoolFileDropTarget::OnDropFiles(wxCoord x, wxCoord y ,const wxArrayString &filenames) {
	if (filenames.Count() > 0) {
		wxString name = filenames.Item(0);
		frame->OpenDroppedFile(name);
	}
    return true;
}

//
// About dialog
//
WavtoolAbout::WavtoolAbout(wxWindow* parent, wxWindowID id)
	: wxDialog(parent, id, _("About..."), wxDefaultPosition, wxDefaultSize, wxDEFAULT_DIALOG_STYLE)
{
	wxSizerFlags flags = wxSizerFlags().Expand().Border(wxALL, 4);

	wxBoxSizer *szrLeft   = new wxBoxSizer(wxVERTICAL);
	wxBoxSizer *szrRight  = new wxBoxSizer(wxVERTICAL);
	wxBoxSizer *szrMain   = new wxBoxSizer(wxHORIZONTAL);
	wxBoxSizer *szrAll    = new wxBoxSizer(wxVERTICAL);

	szrLeft->Add(new wxStaticBitmap(this, wxID_ANY,
		wxBitmap(wavtool_xpm), wxDefaultPosition, wxSize(64, 64))
		, flags);

	wxString str = _T("");
	str += _T("Wavtool, Version ");
	str += _T(APPLICATION_VERSION);
	str += _T(" \"");
	str += _T(PLATFORM);
	str += _T("\"\n\n");
	str	+= _T("using ");
	str += wxVERSION_STRING;
	str += _T("\n\n");
	str	+= _T("Copyright (C) 2011-2015 Sasaji");

	szrRight->Add(new wxStaticText(this, wxID_ANY, str), flags);

	wxSizer *szrButtons = CreateButtonSizer(wxOK);
	szrMain->Add(szrLeft, flags);
	szrMain->Add(szrRight, flags);
	szrAll->Add(szrMain, flags);
	szrAll->Add(szrButtons, flags);

	SetSizerAndFit(szrAll);
}
